@@ -15,6 +15,7 @@ from aiosmb.protocol.smb.command_codes import SMBCommand
 from aiosmb.wintypes.ntstatus import NTStatus
 from aiosmb.protocol.smb.header import SMBHeader, SMBHeaderFlags2Enum
 from aiosmb.protocol.smb.message import SMBMessage
+from aiosmb.protocol.smb.commons import SMBSecurityMode
 from aiosmb.protocol.smb.commands import *
 from aiosmb.protocol.smb2.message import SMB2Message, SMB2Transform, SMB2Compression
 from aiosmb.protocol.smb2.commands.negotiate import SMB2ContextType, SMB2PreauthIntegrityCapabilities, SMB2HashAlgorithm, SMB2Cipher, SMB2CompressionType, SMB2CompressionFlags, SMB2EncryptionCapabilities, SMB2CompressionCapabilities
@@ -398,11 +399,40 @@ class SMBConnection:
 			if err is not None:
 				raise err
 			
-			await self.disconnect()
-			
 			return self.gssapi.get_extra_info(), None
 		except Exception as e:
 			return None, e
+		finally:
+			await self.disconnect()
+
+	async def protocol_test(self, protocol):
+		"""
+		Checks if the remote end supports a given protocol.
+		On success it returns True and the reply from the server (for checking SMB3 capabilities)
+		"""
+		try:
+			self.supported_dialects = protocol
+			_, err = await self.connect()
+			if err is not None:
+				raise err
+			sign_en = None
+			sign_req = None
+			res, rply, err = await self.negotiate(protocol_test = True)
+			if isinstance(rply, SMB2Message):
+				sign_en = NegotiateSecurityMode.SMB2_NEGOTIATE_SIGNING_ENABLED in rply.command.SecurityMode   
+				sign_req = NegotiateSecurityMode.SMB2_NEGOTIATE_SIGNING_REQUIRED in rply.command.SecurityMode   
+			elif isinstance(rply, SMBMessage):
+				sign_en = SMBSecurityMode.NEGOTIATE_SECURITY_SIGNATURES_ENABLED in rply.command.SecurityMode   
+				sign_req = SMBSecurityMode.NEGOTIATE_SECURITY_SIGNATURES_REQUIRED in rply.command.SecurityMode   
+
+			if err is not None:
+				raise err
+
+			return res, sign_en, sign_req, rply, None
+		except Exception as e:
+			return False, None, e
+		finally:
+			await self.disconnect()
 	
 	async def connect(self):
 		"""
@@ -485,7 +515,7 @@ class SMBConnection:
 		ctx.update(self.PreauthIntegrityHashValue + msg_data)
 		self.PreauthIntegrityHashValue = ctx.digest()
 		
-	async def negotiate(self):
+	async def negotiate(self, protocol_test = False):
 		"""
 		Initiates protocol negotiation.
 		First we send an SMB_COM_NEGOTIATE_REQ with our supported dialects
@@ -500,15 +530,23 @@ class SMBConnection:
 				header.Flags    = 0
 				header.Flags2   = SMBHeaderFlags2Enum.SMB_FLAGS2_UNICODE
 					
-				command = SMB_COM_NEGOTIATE_REQ()				
-				command.Dialects = ['SMB 2.???','SMB 2.002']
+				command = SMB_COM_NEGOTIATE_REQ()
+				if protocol_test is True:
+					command.Dialects = ['NT LM 0.12']
+				else:			
+					command.Dialects = ['SMB 2.???','SMB 2.002']
 				
 				msg = SMBMessage(header, command)
 				message_id = await self.sendSMB(msg)
 				#recieveing reply, should be version2, because currently we dont support v1 :(
 				rply, rply_data = await self.recvSMB(message_id, ret_data = True) #negotiate MessageId should be 1
 				if isinstance(rply, SMBMessage):
-					raise Exception('Server replied with SMBv1 message, doesnt support SMBv2')
+					if protocol_test is True:
+						if rply.command.DialectIndex == 65535:
+							return False, rply, None
+						return True, rply, None
+					else:
+						raise Exception('Server replied with SMBv1 message, doesnt support SMBv2')
 				if rply.header.Status != NTStatus.SUCCESS:
 					raise Exception('SMB2 negotiate error! Server replied with error status code!')
 				
@@ -605,9 +643,13 @@ class SMBConnection:
 			#self.ClientSecurityMode = 0			
 			self.status = SMBConnectionStatus.SESSIONSETUP
 			
+			if protocol_test is True:
+				return True, rply, None
 			return True, None
 		
 		except Exception as e:
+			if protocol_test is True:
+				return False, None, None
 			return False, e
 
 	async def session_setup(self, fake_auth = False):
@@ -664,7 +706,7 @@ class SMBConnection:
 			if rply.header.Status == NTStatus.SUCCESS:
 				if self.gssapi.is_guest() is True:
 					self.signing_required = False
-					
+
 				self.SessionKey = self.gssapi.get_session_key()[:16]
 				
 				# TODO: key calc
